
set interfaces bridge br0 description 'Global Switch Bridge'
set interfaces bridge br0 enable-vlan

{% for vname,vdata in vlans.items() if vdata.mode is defined and vdata.mode != 'route' %}
set interfaces bridge br0 vif {{ vdata.id }}
{% endfor +%}
