{#
# Add single interfaces to related bridge
#}
{% for ifdata in interfaces if ifdata.vlan is defined %}

{% if ifdata.vlan.access is defined %}
/interface bridge port add bridge=bridge{{ vlans[ifdata.vlan.access].id }} interface={{ ifdata.ifname }}
{% endif %}
{% if ifdata.vlan.native is defined %}
/interface bridge port add bridge=bridge{{ vlans[ifdata.vlan.native].id }} interface={{ ifdata.ifname }}
{% endif %}

{% endfor %}
