!
router bgp {{ bgp.as }}
 address-family ipv6
!
{% for n in bgp.neighbors if n.ipv4 is defined and n['6pe']|default(False) %}
  neighbor {{ n.ipv4 }} activate
  neighbor {{ n.ipv4 }} send-label
{% endfor %}
