{% set ospf_interfaces = interfaces %}
{% set ospf_main_vrf = True %}
{% set ospf_router_id = ospf.router_id %}

{% if ospf.af.ipv4 is defined %}
{% set instance = "default4" %}
{% set ospf_afi_check = "ipv4" %}
{% set ospf_version = "2" %}
{% include 'routeros7.ospf-include.j2' %}
{% endif %}

{% if ospf.af.ipv6 is defined %}
{% set instance = "default6" %}
{% set ospf_afi_check = "ipv6" %}
{% set ospf_version = "3" %}
{% include 'routeros7.ospf-include.j2' %}
{% endif %}
