    {{ name }}.vm.synced_folder ".", "/vagrant", disabled: true
    {{ name }}.ssh.insert_key = false
    {{ name }}.ssh.extra_args = ["-o", "KexAlgorithms=diffie-hellman-group14-sha1"]
    {{ name }}.vm.boot_timeout = 180
    {{ name }}.vm.guest = :freebsd

    {{ name }}.vm.provider :libvirt do |domain|
      domain.nic_adapter_count = 8
      domain.memory = 2048
      domain.cpus = 1
      domain.driver = "kvm"
    end
