#!/bin/vbash
source /opt/vyatta/etc/functions/script-template

if [ "$(id -g -n)" != 'vyattacfg' ] ; then
    exec sg vyattacfg -c "/bin/vbash $(readlink -f $0) $@"
fi

# Configuration items start here
configure

set bfd profile netsim interval multiplier {{ bfd.multiplier|default(3) }}
set bfd profile netsim interval receive {{ bfd.min_rx|default(500) }}
set bfd profile netsim interval transmit {{ bfd.min_tx|default(500) }}

# Commit, save and exit from subshell

commit
save
exit
