{% if ldp is defined %}
{%   include 'sros.ldp.j2' +%}
{% endif %}
{% if 'bgp' in mpls %}
{%   include 'sros.bgp-lu.j2' +%}
{% endif %}
{% if 'vpn' in mpls %}
{%   include 'sros.mplsvpn.j2' +%}
{% endif %}
{% if '6pe' in mpls %}
{%   include 'sros.6pe.j2' +%}
{% endif %}
